/*****************************************************************************
*                                                                            *
*   Paint.NET Toli Effects                                                   *
*   Copyright (C) Toli Cuturicu 2007-2008                                    *
*   e-Mail: tolicuturicu@yahoo.com                                           *
*                                                                            *
*   This program is free software: you can redistribute it and/or modify     *
*   it under the terms of the GNU General Public License as published by     *
*   the Free Software Foundation, either version 3 of the License, or        *
*   (at your option) any later version.                                      *
*                                                                            *
*   This program is distributed in the hope that it will be useful,          *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of           *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
*   GNU General Public License for more details.                             *
*                                                                            *
*   You should have received a copy of the GNU General Public License        *
*   along with this program.  If not, see <http://www.gnu.org/licenses/>.    *
*                                                                            *
*****************************************************************************/

using System;
using System.Text;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.Drawing;
using System.Reflection;
using System.IO;
using System.Threading;

namespace Toli
{
	public class IniFile
	{

		#region Intro

		// Constants
		public const int BufferLength = 256;
		public const string PaintDotNet = "Paint.NET";
		public const string ToliEffects = "ToliEffects";
		public const string IniExtension = ".ini";
		public const string EnglishSection = "en";
		public const string EncodingSection = "Encoding";
		public const string CodePageKey = "CodePage";
		public const string LeftKey = "Left";
		public const string TopKey = "Top";
		public const string PluginNameKey = "PluginName";
		public const string SubmenuNameKey = "SubmenuName";


		// Fields
		public readonly string FileName;
		public readonly string DefaultSection;
		public readonly Encoding FileEncoding;


		// Constructors
		public IniFile(string FileName, string DefaultSection)
		{
			this.FileName = FileName;
			this.DefaultSection = DefaultSection;
			// Attempts to get the Encoding from the Ini File and sets the default if no information is found
			// There should be an [Encoding] section with a CodePage= key
			this.FileEncoding = Encoding.ASCII;
			int CodePage = this.ReadInteger(EncodingSection, CodePageKey, 0);
			this.FileEncoding = Encoding.GetEncoding(CodePage);
		}

		#endregion


		#region Static

		public static IniFile GetLanguageIniFile()
		{
			string FileName = Path.ChangeExtension(Assembly.GetCallingAssembly().Location, IniExtension);
			string DefaultSection = Thread.CurrentThread.CurrentUICulture.TwoLetterISOLanguageName.ToLowerInvariant();
			return new IniFile(FileName, DefaultSection);
		}

		public static IniFile GetConfigurationIniFile()
		{
			string LocalApplicationDataFolder = Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData) +
				Path.DirectorySeparatorChar + PaintDotNet + Path.DirectorySeparatorChar;
			if (!Directory.Exists(LocalApplicationDataFolder))
				Directory.CreateDirectory(LocalApplicationDataFolder);
			string FileName = LocalApplicationDataFolder + ToliEffects + IniExtension;
			string DefaultSection = Path.GetFileNameWithoutExtension(Assembly.GetCallingAssembly().Location);
			return new IniFile(FileName, DefaultSection);
		}

		#endregion


		#region Private

		[DllImport("kernel32", SetLastError = true)]
		private static extern int GetPrivateProfileString(string Section, string Key, string Default, byte[] ReturnBytes,
			int BufferLength, string FileName);

		[DllImport("kernel32", SetLastError = true)]
		private static extern int WritePrivateProfileString(string Section, string Key, string Value, string FileName);

		private string Read(string Section, string Key, string Default)
		{
			byte[] ReturnBytes = new byte[BufferLength];
			int ByteCount = GetPrivateProfileString(Section, Key, Default, ReturnBytes, BufferLength, this.FileName);
			string Result = this.FileEncoding.GetString(ReturnBytes, 0, ByteCount).TrimEnd((char)0);
			if (Result == string.Empty) return Default;
			return Result;
		}

		private void Write(string Section, string Key, string Value)
		{
			WritePrivateProfileString(Section, Key, Value, this.FileName);
		}

		private static string GetAssemblyTitle(Assembly A)
		{
			object[] Attributes = A.GetCustomAttributes(typeof(AssemblyTitleAttribute), false);
			if (Attributes.Length > 0)
			{
				AssemblyTitleAttribute TitleAttribute = (AssemblyTitleAttribute)Attributes[0];
				if (TitleAttribute.Title != string.Empty) return TitleAttribute.Title;
			}
			return Path.GetFileNameWithoutExtension(A.CodeBase);
		}

		#endregion


		#region String

		public string ReadString(string Section, string Key, string DefaultValue)
		{
			return Read(Section, Key, DefaultValue);
		}

		public string ReadString(string Section, string Key)
		{
			return Read(Section, Key, string.Empty);
		}

		public string ReadString(string Key)
		{
			string EnglishString = Read(EnglishSection, Key, string.Empty);
			return Read(DefaultSection, Key, EnglishString);
		}

		public void WriteString(string Section, string Key, string Value)
		{
			Write(Section, Key, Value);
		}

		public void WriteString(string Key, string Value)
		{
			Write(DefaultSection, Key, Value);
		}

		#endregion


		#region Integer

		public int ReadInteger(string Section, string Key, int DefaultValue)
		{
			int Parsed = 0;
			string AsString = Read(Section, Key, DefaultValue.ToString());
			if (int.TryParse(AsString, out Parsed)) return Parsed;
			else return DefaultValue;
		}

		public int ReadInteger(string Section, string Key)
		{
			return ReadInteger(Section, Key, 0);
		}

		public int ReadInteger(string Key, int DefaultValue)
		{
			return ReadInteger(DefaultSection, Key, DefaultValue);
		}

		public int ReadInteger(string Key)
		{
			return ReadInteger(DefaultSection, Key, 0);
		}


		public void WriteInteger(string Section, string Key, int Value)
		{
			Write(Section, Key, Value.ToString());
		}

		public void WriteInteger(string Key, int Value)
		{
			Write(DefaultSection, Key, Value.ToString());
		}

		#endregion


		#region Boolean

		public bool ReadBoolean(string Section, string Key, bool DefaultValue)
		{
			bool Parsed = false;
			string AsString = Read(Section, Key, DefaultValue.ToString());
			if (bool.TryParse(AsString, out Parsed)) return Parsed;
			else return DefaultValue;
		}

		public bool ReadBoolean(string Section, string Key)
		{
			return ReadBoolean(Section, Key, false);
		}

		public bool ReadBoolean(string Key, bool DefaultValue)
		{
			return ReadBoolean(DefaultSection, Key, DefaultValue);
		}

		public bool ReadBoolean(string Key)
		{
			return ReadBoolean(DefaultSection, Key, false);
		}


		public void WriteBoolean(string Section, string Key, bool Value)
		{
			Write(Section, Key, Value.ToString());
		}

		public void WriteBoolean(string Key, bool Value)
		{
			Write(DefaultSection, Key, Value.ToString());
		}

		#endregion


		#region Form

		public void ReadFormLocation(string Section, Form Value)
		{
			Value.Left = ReadInteger(Section, LeftKey, Value.Left);
			Value.Top = ReadInteger(Section, TopKey, Value.Top);
		}

		public void ReadFormLocation(Form Value)
		{
			ReadFormLocation(DefaultSection, Value);
		}

		public void WriteFormLocation(string Section, Form Value)
		{
			WriteInteger(Section, LeftKey, Value.Left);
			WriteInteger(Section, TopKey, Value.Top);
		}

		public void WriteFormLocation(Form Value)
		{
			WriteFormLocation(DefaultSection, Value);
		}

		#endregion


		#region Resources

		public string ReadPluginName()
		{
			Assembly PluginAssembly = Assembly.GetCallingAssembly();
			return ReadString(DefaultSection, PluginNameKey, GetAssemblyTitle(PluginAssembly));
		}

		public string ReadSubmenuName()
		{
			string EnglishSubmenuName = ReadString(EnglishSection, SubmenuNameKey, string.Empty);
			return ReadString(DefaultSection, SubmenuNameKey, EnglishSubmenuName);
		}

		public void ReadControlText(Control Ctrl)
		{
			string Text = ReadString(DefaultSection, Ctrl.Name, Ctrl.Text);
			Ctrl.Text = Text.Replace("\\\\", Environment.NewLine);
		}

		#endregion

	}
}
